#pragma once

#include <Engine/ska/Skeleton.h>
#include <Engine/Ska/AnimSet.h>
#include <Engine/Templates/StaticStackArray.h>
#include <Engine/Templates/StaticArray.h>
#include <Engine/templates/DynamicContainer.h>



struct MeshInstance
{
  CMesh *mi_pMesh;
  CStaticArray<struct TextureInstance> mi_tiTextures;
};

struct TextureInstance
{
  int ti_iTextureID;
  CTextureData *ti_tdTexture;
};

struct AnimQueue
{
  CStaticStackArray<struct AnimList> aq_Lists;
};

struct AnimList
{
  FLOAT al_fStartTime;
  FLOAT al_fFadeTime;
  CStaticStackArray<struct PlayedAnim> al_PlayedAnims;
};

struct PlayedAnim
{
  FLOAT pa_fStartTime;
  INDEX pa_iAnimID;
  ULONG pa_ulFlags;
  FLOAT pa_Strength;
  INDEX pa_GroupID;
};

class ENGINE_API CModelInstance
{
public:
  CModelInstance();
  ~CModelInstance();
  void AddMesh(CTFileName fnMesh);
  void AddSkeleton(CTFileName fnSkeleton);
  void AddAnimSet(CTFileName fnAnimSet);
  void AddTexture(CTFileName fnTexture, CTString strTexID);
  void AddAnimation(INDEX iAnimID, DWORD dwFlags, FLOAT fStrength,INDEX iGroupID);
  CModelInstance *AddChild(INDEX iParentBoneID, FLOAT (&fOffset)[6]);
  void NewClonedState(FLOAT fFadeTime);
  void NewClearState(FLOAT fFadeTime);
  void RemovePassedAnimsFromQueue();
  void RemAnimation(INDEX iAnimID);
  void RemAnimsWithID(INDEX iGroupID);
  void Clear();

  CSkeleton *mi_psklSkeleton;                             // pointer to Skeleton
  CDynamicContainer<class CAnimSet> mi_aAnimSet;          // array of animsets
  CStaticArray<struct MeshInstance> mi_aMeshInst;         // array of mesh instances
  CDynamicContainer<class CModelInstance> mi_cmiChildren; // childeren of this model instance

  AnimQueue mi_aqAnims;           // Queue for animations
  INDEX mi_iParentBoneID;      // bone to attach to in parent model's skeleton
  COLOR mi_colModelColor;       // color
  FLOAT3D mi_vStretch;          // stretch
  QVect mi_qvOffset;        // bone offset
}; 

ENGINE_API CModelInstance *ParseSmcFile(CTString &fnAnimFile);
