#pragma once
#include <Engine/Math/Vector.h>
#include <Engine/Math/Quaternion.h>
#include <Engine/Ska/ModelInstance.h>
#include <Engine/Ska/Skeleton.h>
#include <Engine/Ska/Mesh.h>

#define RMF_WIREFRAME           (1UL<<0) // set wireframe on
#define RMF_SHOWSKELETON        (1UL<<1) // show skeleton
#define RMF_SHOWTEXTURE         (1UL<<2) // show texture

#define AN_LOOPING              (1UL<<0)
#define AN_BLEND                (1UL<<1)
#define AN_ADD                  (1UL<<2)

// Rendering structures
struct RenModel
{
  CModelInstance *rm_pmiModel;
  INDEX rm_iParentModelIndex; // index of parent in _aRenModel array
  INDEX rm_iParentBoneIndex; 
  float rm_mTransform[12];    // for models without skeletons 
  
  INDEX rm_iFirstBone;        // index in array _aRenBones where is this RenModel's first bone
  INDEX rm_ctBones;           // how much bones in array
  INDEX rm_iFirstMesh;
  INDEX rm_ctMeshes;
  INDEX rm_iFirstChildModel;
  INDEX rm_iNextSiblingModel;

  INDEX rm_iSkeletonLODIndex; // curent LOD index of slod_aBones array in Skeleton

};

struct RenBone
{
  SkeletonBone *rb_psbBone;
  INDEX rb_iParentIndex;
  INDEX rb_iRenModelIndex;
  QVect rb_qvRelPlacement;
  float rb_mTransform[12];
  float rb_aBonePos[3]; // position for rendering bone
};

struct RenMorph
{
  MeshMorphMap *rmp_pmmmMorphMap;
  FLOAT rmp_fFactor;
};

struct RenWeight
{
  MeshWeightMap *rw_pwmWeightMap;
  INDEX rw_iBoneIndex;
};

struct RenMesh
{
  struct MeshInstance *rmsh_pMeshInst;
  INDEX rmsh_iRenModelIndex;
  INDEX rmsh_iFirstWeight;
  INDEX rmsh_ctWeights;
  INDEX rmsh_iFirstMorph;
  INDEX rmsh_ctMorphs;
  INDEX rmsh_iMeshLODIndex;           // curent LOD index of msh_aMeshLODs array in Mesh
};

// initialize batch model rendering
ENGINE_API void RM_BeginRenderingView(CAnyProjection3D &apr, CDrawPort *pdp);
// cleanup after batch model rendering
ENGINE_API void RM_EndRenderingView(void);
// setup light parameters
ENGINE_API void RM_SetLightDirection(const FLOAT3D &vLightDir);
ENGINE_API void RM_SetLightColor(COLOR colAmbient, COLOR colLight);
// LOD factor management
ENGINE_API void RM_SetDistanceFactor(FLOAT fDistFactor);
ENGINE_API FLOAT RM_GetMipFactor(void);
// setup object position
ENGINE_API void RM_SetObjectPlacement(const CPlacement3D &pl);
ENGINE_API void RM_SetObjectPlacement(const FLOATmatrix3D &m, const FLOAT3D &v);
// render one SKA model with its children
ENGINE_API void RM_RenderSKA(CModelInstance &mi);

// Matrix12 operations
ENGINE_API void Matrix12ToQVect(const FLOAT (&m12)[12], QVect &qv);
ENGINE_API void MultiplyMatrices12(const FLOAT (&m)[12], const FLOAT (&n)[12], FLOAT (&conc)[12]);

// model flags
ENGINE_API void RM_SetFlags(ULONG ulNewFlags);
ENGINE_API ULONG RM_GetFlags();
ENGINE_API void RM_AddFlag(ULONG ulFlag);
ENGINE_API void RM_RemoveFlag(ULONG ulFlag);
